"""
ScreenBuffer
"""

EMPTY_TUPLE = ('','','')
INTRO_BUFFER = [
	('#101011','[multitail]',"               __ __   _  __         _  __"),
	('#101000','[multitail]',"  __ _  __ __ / // /_ (_)/ /_ ___ _ (_)/ /"),
	('#111011','[multitail]'," /  ' \/ // // // __// // __// _ `// // / "),
	('#101011','[multitail]',"/_/_/_/\_,_//_/ \__//_/ \__/ \_,_//_//_/  "),
	('#101011','[multitail]',"~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~")
]

class ScreenBuffer:
	def __init__(self, get_config):
		self._get_config = get_config
		self.__buffer = []
		self.__offset = 0
		self.__changed = True
	
	def showIntro(self):
		self.__buffer.extend(INTRO_BUFFER)
		self.__changed = True
	
	def __slide(self):
		bl = self._get_config("scroll_length")+self._get_config("length")
		if bl==0:
			self.__buffer=[]
		elif len(self.__buffer)>bl:
			self.__buffer = self.__buffer[-int(bl):]
			
	def getSize(self):
		return len(self.__buffer)
	
	def isChanged(self):
		return self.__changed
	
	def scroll_up(self):
		if self.__offset+self._get_config("length")>=len(self.__buffer):
			return
		self.__offset = self.__offset + 1
		self.__changed = True
	
	def scroll_down(self):
		if self.__offset>0:
			self.__offset = self.__offset - 1
			self.__changed = True
		else:
			self.__offset = 0
	
	def put(self, color, name, line):
		self.__buffer.append((color, name, line))
		self.__slide()
		self.__changed = True
		if self._get_config("scroll_jump"):
			self.__offset = 0
		elif not self.__offset==0:
			self.scroll_up()
	
	def list(self, length):
		self.__changed = False
		cb = len(self.__buffer)
		if cb==0:
			return [EMPTY_TUPLE]*length
		
		nr = cb-length
		if nr<0:
			return self.__buffer[:]+([EMPTY_TUPLE]*-nr)
		else:
			if self.__offset==0:
				return self.__buffer[nr:]
			else:
				return self.__buffer[nr-self.__offset:-self.__offset]
