"""
TailBook.py
"""

from xml.sax.saxutils import escape
from stat import *
import os
import os.path

class Tail:
	fn = None
	path = ""
	name = ""
	state = ""
	color = "#000000"

class TailBook:
	def __init__(self):
		self.__files = {}
		
	def __open(self, path, color="#000000"):
		t = Tail()
		t.path = path
		t.color = color
		t.name = os.path.basename(path)
		try:
			t.stat = os.stat(path)
			t.fn = open(path, "r")
			t.fn.seek(t.stat[ST_SIZE])
			t.state = "ok"
		except Exception, e:
			t.state = "%s: '%s'" % (e.strerror, e.filename)
			
		self.__files[path] = t

	def __close(self, path):
		if self.__files.has_key(path):
			t = self.__files[path]
			try:
				t.fn.close()
			except Exception, e:
				print "Could not close file %: %s" % (e.strerror, e.filename)
			del self.__files[path]
	
	def update(self, list, colors):
		assert len(list)==len(colors), "TailBook: nr of colors and files must be equal"
		
		# open new
		for n in list:
			if not self.__files.has_key(n):
				self.__open(n)
				
		# close old 
		for o in self.__files.keys():
			if not o in list:
				self.__close(o)
		
		# change colors
		for i in range(len(list)):
			key = list[i]
			t = self.__files[key]
			if not t.color == colors[i]:
				t.color = colors[i]
				self.__files[key]=t
	
	def check(self):
		""" check log files for resets """
		for key in self.__files.keys():
			t = self.__files[key]
			if not t.state=="ok":
				continue
				
			try:
				stat = os.stat(t.path)
			except Exception, e:
				t.state = "%s: '%s'" % (e.strerror, e.filename)
				self.__files[key] = t				
				continue # error reaching file
				
			if t.stat[ST_INO]!=stat[ST_INO]:
				# file inode changed
				print "inode changed for %s" % t.path
				self.__close(key)
				self.__open(t.path, t.color)
				continue
			elif t.stat[ST_SIZE]>stat[ST_SIZE]:
				# size went backwards
				# on 256 bytes try to rollback one line
				print "size went backwards in %s" % t.path
				try:
					BUFFER = 256
					pos = stat[ST_SIZE]-BUFFER
					if pos<0: pos=0
					t.fn.seek(pos)
					chunk = t.fn.read(BUFFER).strip()
					rb = chunk.rfind('\n')
					if rb==-1:
						# no break, then start at the end
						t.fn.seek(stat[ST_SIZE])
					else:
						# point to the rollbacked line
						t.fn.seek(pos+rb)
				except:
					print "could not set position on file %s" % t.path
					continue # error reaching file, deal with it later
			
			self.__files[key].stat=stat # update stat
		
		
	def getList(self):
		return self.__files.keys()
		
	def getPool(self):
		return [key for key in self.__files.keys() if self.__files[key].state=="ok"]
	
	def getState(self, key):
		if self.__files.has_key(key):
			return self.__files[key].state
		else:
			return "state does not exists"
		
	def getName(self, key):
		if self.__files.has_key(key):
			return self.__files[key].name
		return ""
	
	def getColor(self, key):
		if self.__files.has_key(key):
			return self.__files[key].color
		return ""
	
	def getLine(self, key):
		if self.__files.has_key(key):
			t = self.__files[key]
			
			if not t.state=="ok":
				return ""
				
			try:
				return escape(t.fn.readline().replace('\n','').replace('\r',''))
			except Exception, e:
				t.state = "%s: '%s'" % (e.strerror, e.filename)
				self.__files[key] = t
				return t.state
